##############################################################################
#
# LICENSE:
# This code is bound to be ugly, have mistakes,
# and generally just be plain _wrong_.
#
# What do you care though?  It works, and you
# got it for free. ;-)  Feel free to make any
# changes and re-releases.
#
# Additionally it complies with the terms of the Zope Public License,
# Version 2.0 (ZPL)
#
##############################################################################
__doc__ = """Zope Example Classes with O2DB Support by (-:fs)"""
__version__ = '$Revision: 0.5.7 $'[11:-2]

from AccessControl import ClassSecurityInfo
from Globals import InitializeClass, DTMLFile
from OFS.SimpleItem import SimpleItem
from OFS.PropertyManager import PropertyManager

from zopeAssists import O2DBbase, getCB
from o2dbMain import *


## -----------------------------------------------------------------------------
## ----------------------------------  Book  -----------------------------------
## -----------------------------------------------------------------------------
manage_addBookForm = DTMLFile('dtml/manage_addBookForm', globals())
def manage_addBook(self, bookAuthor, bookTitle, bookPublisher='',
                   bookYearOfRelease='', bookISBN='', REQUEST=None):
    """add a book object to the external relational DB via the O2DB interface
       AND author and title as 'title' of the zope object to ZODB
    """
    # create external object in relational DB
    extObj = PersistentObject(getCB())
    extObj.addAttribute("CHAR", "objectcategory", "Book")
    extObj.addAttribute("CHAR", "Author", bookAuthor)
    extObj.addAttribute("CHAR", "Title", bookTitle)
    extObj.addAttribute("CHAR", "Publisher", bookPublisher)
    extObj.addAttribute("CHAR", "Year of Release", bookYearOfRelease)
    extObj.addAttribute("CHAR", "ISBN", bookISBN)
    extObj.save()
    
    # create zope object
    # parameters of constructor - id, title, externalObjectID
    # id    = objectcatgegory and external object ID
    # title = choose some descriptive attributes of the object
    id = extObj.getAttributeValue("objectcategory") + ' - ' + extObj.getID()
    obj = Book(id,
               bookAuthor + ": '" + bookTitle + "'",
               extObj.getID())
    self._setObject(id, obj)

    if REQUEST is not None: return self.manage_main(self, REQUEST)

class Book(O2DBbase):   
    """Book stored in relational DB via the O2DB interface"""
    meta_type = 'Book'
    security = ClassSecurityInfo()
    
    manage_options = (
        (
          {'label': 'Edit Book', 'action': 'manage_edit',
           'help': ('O2DB','Book.stx') }, )
          + PropertyManager.manage_options
          + SimpleItem.manage_options )

    security.declareProtected('View management screens', 'manage_main')
    manage_main = DTMLFile('dtml/manage_editBookForm', globals())

    security.declareProtected('View management screens', 'manage_edit')
    def manage_edit(self, id='', title='', bookAuthor='', bookTitle='', bookPublisher='',
                    bookYearOfRelease='', bookISBN='', REQUEST=None):
        """edit a book from the external relational DB via the O2DB interface"""
        ## get external object from mySQL with extObjID
        extObj = PersistentObject(getCB(), self.extObjID)

        attrAuthor = extObj.getAttribute('Author')
        if not bookAuthor: bookAuthor = attrAuthor.getValue()
        attrTitle = extObj.getAttribute('Title')
        if not bookTitle: bookTitle = attrTitle.getValue()
        attrPublisher = extObj.getAttribute('Publisher')
        if not bookPublisher: bookPublisher = attrPublisher.getValue()
        attrYearOfRelease = extObj.getAttribute('Year of Release')
        if not bookYearOfRelease: bookYearOfRelease = attrYearOfRelease.getValue()
        attrISBN = extObj.getAttribute('ISBN')
        if not bookISBN: bookISBN = attrISBN.getValue()
        
        if REQUEST is not None:
            attrAuthor.setValue(bookAuthor)
            attrTitle.setValue(bookTitle)
            attrPublisher.setValue(bookPublisher)
            attrYearOfRelease.setValue(bookYearOfRelease)
            attrISBN.setValue(bookISBN)
            extObj.save()
            
            self.title = bookAuthor + ": '" + bookTitle + "'"

            REQUEST.set('bookAuthor', bookAuthor)
            REQUEST.set('bookTitle', bookTitle)
            REQUEST.set('bookPublisher', bookPublisher)
            REQUEST.set('bookYearOfRelease', bookYearOfRelease)
            REQUEST.set('bookISBN', bookISBN)
            try:
                REQUEST['submit']
                return self.manage_main(self, REQUEST,
                                        manage_tabs_message=self.save_message)
            except KeyError:  return self.manage_main(self, REQUEST)
    
InitializeClass(Book)


## -----------------------------------------------------------------------------
## ---------------------------------  Contact  ---------------------------------
## -----------------------------------------------------------------------------
manage_addContactForm = DTMLFile('dtml/manage_addContactForm', globals())
def manage_addContact(self, ctLastName, ctFirstName='', ctMobilePhone='', \
                      ctEmail='', ctCompany='', REQUEST=None):
    """add a contact object to the external relational DB via the O2DB interface
       AND surname and lastname as 'title' of the zope object to ZODB
    """
    # create external object in relational DB
    extObj = PersistentObject(getCB())
    extObj.addAttribute("CHAR", "objectcategory", "Contact")
    extObj.addAttribute("CHAR", "Last Name", ctLastName)
    extObj.addAttribute("CHAR", "First Name", ctFirstName)
    extObj.addAttribute("CHAR", "Mobile Phone", ctMobilePhone)
    extObj.addAttribute("CHAR", "Email", ctEmail)
    extObj.addAttribute("CHAR", "Company", ctCompany)
    extObj.save()
    
    # create zope object
    id = extObj.getAttributeValue("objectcategory") + ' - ' + extObj.getID()
    obj = Contact(id,
                  ctLastName + ' ' + ctFirstName,
                  extObj.getID())
    self._setObject(id, obj)
    
    if REQUEST is not None: return self.manage_main(self, REQUEST)

    
class Contact(O2DBbase):
    """Contact stored in relational DB via the O2DB interface"""
    meta_type = 'Contact'
    security = ClassSecurityInfo()
    
    manage_options = (
        ({'label': 'Edit Contact', 'action': 'manage_edit',
          'help': ('O2DB','Contact.stx')}, )
          + PropertyManager.manage_options
          + SimpleItem.manage_options )

    #security.declareProtected('View management screens', 'manage_search')
    manage_search = DTMLFile('dtml/manage_searchContactForm', globals())
    #security.declareProtected('View management screens', 'manage_main')
    manage_main = DTMLFile('dtml/manage_editContactForm', globals())

    security.declareProtected('View management screens', 'manage_edit')
    def manage_edit(self, id='', title='', ctLastName='', ctFirstName='', ctMobilePhone='', \
                    ctEmail='', ctCompany='', REQUEST=None):
        """edit a contact from the external relational DB via the O2DB interface"""       
        ## get external object from mySQL with extObjID
        extObj = PersistentObject(getCB(), self.extObjID)
        
        attrLastName = extObj.getAttribute('Last Name')
        if not ctLastName: ctLastName = attrLastName.getValue()
        attrFirstName = extObj.getAttribute('First Name')
        if not ctFirstName: ctFirstName = attrFirstName.getValue()
        attrMobilePhone = extObj.getAttribute('Mobile Phone')
        if not ctMobilePhone: ctMobilePhone = attrMobilePhone.getValue()
        attrEmail = extObj.getAttribute('Email')
        if not ctEmail: ctEmail = attrEmail.getValue()
        attrCompany = extObj.getAttribute('Company')
        if not ctCompany: ctCompany = attrCompany.getValue()
                
        if REQUEST is not None:
            attrLastName.setValue(ctLastName)
            attrFirstName.setValue(ctFirstName)
            attrMobilePhone.setValue(ctMobilePhone)
            attrEmail.setValue(ctEmail)
            attrCompany.setValue(ctCompany)
            extObj.save()

            self.title = ctLastName + ' ' + ctFirstName
            
            REQUEST.set('ctLastName', ctLastName)
            REQUEST.set('ctFirstName', ctFirstName)
            REQUEST.set('ctMobilePhone', ctMobilePhone)
            REQUEST.set('ctEmail', ctEmail)
            REQUEST.set('ctCompany', ctCompany)
            try:
                REQUEST['submit']
                return self.manage_main(self, REQUEST,
                                        manage_tabs_message=self.save_message)
            except KeyError:  return self.manage_main(self, REQUEST)
            
InitializeClass(Contact)


## -----------------------------------------------------------------------------
## -------------------------------  SoundCarrier  ------------------------------
## -----------------------------------------------------------------------------
manage_addSoundCarrierForm = DTMLFile('dtml/manage_addSoundCarrierForm', globals())
def manage_addSoundCarrier(self, scArtist, scTitle, scType='', REQUEST=None):
    """add a SoundCarrier to the external relational DB via the O2DB interface
       AND to ZODB
    """
    # create external object in relational DB
    extObj = PersistentObject(getCB())
    extObj.addAttribute("CHAR", "objectcategory", "Sound Carrier")
    extObj.addAttribute("CHAR", "Artist", scArtist)
    extObj.addAttribute("CHAR", "Title", scTitle)
    extObj.addAttribute("CHAR", "Type", scType)
    extObj.save()
    
    # create zope object
    id = scType + ' - ' + extObj.getID()
    obj = SoundCarrier(id,
                       scArtist + ": " + scTitle,
                       extObj.getID())
    self._setObject(id, obj)
    
    if REQUEST is not None: return self.manage_main(self, REQUEST)

    
class SoundCarrier(O2DBbase):
    """Contact stored in relational DB via the O2DB interface"""
    meta_type = 'Sound Carrier'
    security = ClassSecurityInfo()
    
    manage_options = (
        ({'label': 'Edit Sound Carrier', 'action': 'manage_edit',
          'help': ('O2DB','SoundCarrier.stx') }, )
          + PropertyManager.manage_options
          + SimpleItem.manage_options )

    #security.declareProtected('View management screens', 'manage_search')
    manage_search = DTMLFile('dtml/manage_searchForm', globals())
    #security.declareProtected('View management screens', 'manage_main')
    manage_main = DTMLFile('dtml/manage_editSoundCarrierForm', globals())
     
    security.declareProtected('View management screens', 'manage_edit')
    def manage_edit(self, id='', title='', scArtist='', scTitle='', scType='', REQUEST=None):
        """edit a contact from the external relational DB via the O2DB interface"""
        ## get external object from mySQL with extObjID        
        extObj = PersistentObject(getCB(), self.extObjID)
        
        attrArtist = extObj.getAttribute('Artist')
        if not scArtist: scArtist = attrArtist.getValue()
        attrTitle = extObj.getAttribute('Title')
        if not scTitle: scTitle = attrTitle.getValue()
        attrType = extObj.getAttribute('Type')
        if not scType: scType = attrType.getValue()

        if REQUEST is not None:
            attrArtist.setValue(scArtist)
            attrTitle.setValue(scTitle)
            attrType.setValue(scType)
            extObj.save()

            self.title = scArtist + ": " + scTitle
            
            REQUEST.set('scArtist', scArtist)
            REQUEST.set('scTitle', scTitle)
            REQUEST.set('scType', scType)
            try:
                REQUEST['submit']
                return self.manage_main(self, REQUEST,
                                        manage_tabs_message=self.save_message)
            except KeyError:  return self.manage_main(self, REQUEST)
            
InitializeClass(SoundCarrier)
